@@ -69,7 +69,7 @@ class LibLineReader(LineReader):
     def get_line(self):
         res = self.f.readline()
         while res and res[0] == '#':
-            if res.startswith('#End Library'):
+            if res.startswith('#End Library') or res.startswith('# End Library'):
                 return res.rstrip()
             self.line += 1
             res = self.f.readline()
@@ -635,7 +635,7 @@ class SymLib(object):
             if not line.startswith('EESchema-LIBRARY'):
                 raise SchLibError('Missing library signature', line, f)
             line = f.get_line()
-            while not line.startswith('#End Library'):
+            while not (line.startswith('#End Library') or line.startswith('# End Library')):
                 if line.startswith('DEF'):
                     o = LibComponent(line, f, file)
                     if o.name:
@@ -746,6 +746,9 @@ class SchematicField(object):
             f.write(' "{}"'.format(self.name))
         f.write('\n')
 
+    def __str__(self):
+        return self.name+'='+self.value
+
 
 class SchematicAltRef():
     def __init__(self):
@@ -798,6 +801,8 @@ class SchematicComponent(object):
                   - BoM DNF: only if True (and fitted is False)
         - fixed: means you can't change it by a replacement without authorization
                  Is just a flag and doesn't affect much.
+        - footprint_rot: angle to rotate the part in the pick & place.
+        - qty: ammount of this part used.
         """
     ref_re = re.compile(r'([^\d]+)([\?\d]+)')
 
@@ -818,6 +823,7 @@ class SchematicComponent(object):
         self.fixed = False
         self.bottom = False
         self.footprint_rot = 0.0
+        self.qty = 1
         # KiCad 5 PCB flags (mutually exclusive)
         self.smd = False
         self.virtual = False
@@ -829,6 +835,9 @@ class SchematicComponent(object):
             return self.dfields[field].value
         return ''
 
+    def is_field(self, field):
+        return field in self.dfields
+
     def get_free_field_number(self):
         """ Looks for a field number that isn't currently in use """
         max_num = -1
@@ -839,8 +848,9 @@ class SchematicComponent(object):
 
     def set_field(self, field, value):
         """ Change the value for an existing field """
-        if field in self.dfields:
-            target = self.dfields[field]
+        field_lc = field.lower()
+        if field_lc in self.dfields:
+            target = self.dfields[field_lc]
             target.value = value
             # Adjust special fields
             if target.number < 4:
@@ -852,8 +862,9 @@ class SchematicComponent(object):
             f.number = self.get_free_field_number()
             self.add_field(f)
 
-    # def get_field_names(self):
-    #     return [f.name for f in self.fields]
+    def get_field_names(self):
+        """ List of all the available field names for this component """
+        return self.dfields.keys()
 
     def get_user_fields(self):
         """ Returns a list of tuples with the user defined fields (name, value) """
@@ -863,6 +874,13 @@ class SchematicComponent(object):
         self.fields.append(field)
         self.dfields[field.name.lower()] = field
 
+    def rename_field(self, old_name, new_name):
+        old_name = old_name.lower()
+        field = self.dfields[old_name]
+        field.name = new_name
+        del self.dfields[old_name]
+        self.dfields[new_name.lower()] = field
+
     def back_up_fields(self):
         """ First call makes a back-up of the fields.
             Next calls restores the back-up. """
@@ -1148,15 +1166,34 @@ class SchematicWire(object):
     ENTRIES = {'Wire': ENTRY_WIRE, 'Bus': ENTRY_BUS}
     NAMES = ['Wire Wire Line', 'Wire Bus Line', 'Wire Notes Line', 'Entry Wire Line', 'Entry Bus Bus']
 
-    def __init__(self):
+    def __init__(self, width=None, style=None, rgb=None):
         super().__init__()
+        self.width = width
+        self.rgb = rgb
+        self.style = style
 
     @staticmethod
     def load(f, line):
         res = _split_space(line)
-        if len(res) != 3:
+        res_l = len(res)
+        width = style = rgb = None
+        if res_l > 3 and res[0] == 'Wire' and res[1] == 'Notes' and res[2] == 'Line':
+            offset = 3
+            while offset < res_l:
+                if res[offset] == 'width' and res_l > offset+1:
+                    width = res[offset+1]
+                    offset += 2
+                elif res[offset] == 'style' and res_l > offset+1:
+                    style = res[offset+1]
+                    offset += 2
+                elif res[offset].startswith('rgb(') and res_l > offset+2:
+                    rgb = res[offset]+' '+res[offset+1]+' '+res[offset+2]
+                    offset += 3
+                else:
+                    raise SchFileError('Malformed wire note', line, f)
+        elif res_l != 3:
             raise SchFileError('Malformed wire', line, f)
-        wire = SchematicWire()
+        wire = SchematicWire(width, style, rgb)
         if res[0] == 'Wire':
             # Wire Wire Line
             # Wire Bus Line
@@ -1183,7 +1220,14 @@ class SchematicWire(object):
         return wire
 
     def write(self, f):
-        f.write(SchematicWire.NAMES[self.type])
+        extra = ''
+        if self.width is not None:
+            extra += ' width '+self.width
+        if self.style is not None:
+            extra += ' style '+self.style
+        if self.rgb is not None:
+            extra += ' '+self.rgb
+        f.write(SchematicWire.NAMES[self.type]+extra)
         f.write('\n\t{} {} {} {}\n'.format(self.x, self.y, self.ex, self.ey))
 
 
@@ -1474,7 +1518,7 @@ class Schematic(object):
         files = {self.fname}
         for sch in self.sheets:
             files.update(sch.sheet.get_files())
-        return list(files)
+        return sorted(files)
 
     def get_components(self, exclude_power=True):
         """ A list of all the components. """
