@@ -22,7 +22,8 @@ from collections import OrderedDict
 from .gs import GS
 from .misc import (PLOT_ERROR, MISSING_TOOL, CMD_EESCHEMA_DO, URL_EESCHEMA_DO, CORRUPTED_PCB,
                    EXIT_BAD_ARGS, CORRUPTED_SCH, EXIT_BAD_CONFIG, WRONG_INSTALL, UI_SMD, UI_VIRTUAL, KICAD_VERSION_5_99,
-                   MOD_SMD, MOD_THROUGH_HOLE, MOD_VIRTUAL, W_PCBNOSCH, W_NONEEDSKIP, W_WRONGCHAR, name2make, W_TIMEOUT)
+                   MOD_SMD, MOD_THROUGH_HOLE, MOD_VIRTUAL, W_PCBNOSCH, W_NONEEDSKIP, W_WRONGCHAR, name2make, W_TIMEOUT,
+                   W_KIAUTO)
 from .error import PlotError, KiPlotConfigurationError, config_error, trace_dump
 from .pre_base import BasePreFlight
 from .kicad.v5_sch import Schematic, SchFileError
@@ -131,6 +132,34 @@ def search_as_plugin(cmd, names):
     return cmd
 
 
+def extract_errors(text):
+    in_error = in_warning = False
+    msg = ''
+    for line in text.split('\n'):
+        line += '\n'
+        if line[0] == ' ' and (in_error or in_warning):
+            msg += line
+        else:
+            if in_error:
+                in_error = False
+                logger.error(msg.rstrip())
+            elif in_warning:
+                in_warning = False
+                logger.warning(W_KIAUTO+msg.rstrip())
+        if line.startswith('ERROR:'):
+            in_error = True
+            msg = line[6:]
+        elif line.startswith('WARNING:'):
+            in_warning = True
+            msg = line[8:]
+    if in_error:
+        in_error = False
+        logger.error(msg.rstrip())
+    elif in_warning:
+        in_warning = False
+        logger.warning(W_KIAUTO+msg.rstrip())
+
+
 def exec_with_retry(cmd):
     logger.debug('Executing: '+str(cmd))
     retry = 2
@@ -141,11 +170,9 @@ def exec_with_retry(cmd):
         if ret > 0 and ret < 128 and retry:
             logger.debug('Failed with error {}, retrying ...'.format(ret))
         else:
+            extract_errors(result.stderr)
             err = '> '+result.stderr.replace('\n', '\n> ')
-            if ret:
-                logger.error('Output from command:\n'+err)
-            else:
-                logger.debug('Output from command:\n'+err)
+            logger.debug('Output from command:\n'+err)
             if 'Timed out' in err:
                 logger.warning(W_TIMEOUT+'Time out detected, on slow machines or complex projects try:')
                 logger.warning(W_TIMEOUT+'`kiauto_time_out_scale` and/or `kiauto_wait_start` global options')
@@ -281,12 +308,12 @@ def preflight_checks(skip_pre):
     BasePreFlight.run_enabled()
 
 
-def get_output_dir(o_dir):
+def get_output_dir(o_dir, dry=False):
     # outdir is a combination of the config and output
     outdir = os.path.abspath(os.path.join(GS.out_dir, o_dir))
     # Create directory if needed
     logger.debug("Output destination: {}".format(outdir))
-    if not os.path.exists(outdir):
+    if not dry and not os.path.exists(outdir):
         os.makedirs(outdir)
     return outdir
 
@@ -299,7 +326,7 @@ def config_output(out, dry=False):
         if out.is_sch():
             load_sch()
     try:
-        out.config()
+        out.config(None)
     except KiPlotConfigurationError as e:
         config_error("In section '"+out.name+"' ("+out.type+"): "+str(e))
 
@@ -307,7 +334,8 @@ def config_output(out, dry=False):
 def run_output(out):
     GS.current_output = out.name
     try:
-        out.run(get_output_dir(out.dir))
+        out_dir = out.expand_dirname(out.dir)
+        out.run(get_output_dir(out_dir))
         out._done = True
     except PlotError as e:
         logger.error("In output `"+str(out)+"`: "+str(e))
@@ -367,41 +395,10 @@ def gen_global_targets(f, pre_targets, out_targets, type):
     return extra_targets
 
 
-def generate_makefile(makefile, cfg_file, outputs, kibot_sys=False):
-    cfg_file = os.path.relpath(cfg_file)
-    logger.info('- Creating makefile `{}` from `{}`'.format(makefile, cfg_file))
-    with open(makefile, 'wt') as f:
-        f.write('#!/usr/bin/make\n')
-        f.write('# Automatically generated by KiBot from `{}`\n'.format(cfg_file))
-        fname = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'src', 'kibot'))
-        if kibot_sys or not os.path.isfile(fname):
-            fname = 'kibot'
-        f.write('KIBOT={}\n'.format(fname))
-        dbg = ''
-        if GS.debug_level > 0:
-            dbg = '-'+'v'*GS.debug_level
-        f.write('DEBUG={}\n'.format(dbg))
-        f.write('CONFIG={}\n'.format(cfg_file))
-        f.write('SCH={}\n'.format(os.path.relpath(GS.sch_file)))
-        f.write('PCB={}\n'.format(os.path.relpath(GS.pcb_file)))
-        f.write('DEST={}\n'.format(os.path.relpath(GS.out_dir)))
-        f.write('KIBOT_CMD=$(KIBOT) $(DEBUG) -c $(CONFIG) -e $(SCH) -b $(PCB) -d $(DEST)\n')
-        f.write('LOGFILE=kibot_error.log\n')
-        f.write('\n')
-        # Configure all outputs
-        GS.outputs = outputs
-        for out in outputs:
-            config_output(out)
-        # Get all targets and dependencies
-        targets = OrderedDict()
-        dependencies = OrderedDict()
-        comments = {}
-        ori_names = {}
-        is_pre = set()
-        pre_pcb_targets = pre_sch_targets = ''
-        out_pcb_targets = out_sch_targets = ''
-        # Preflights
-        pres = BasePreFlight.get_in_use_objs()
+def get_pre_targets(targets, dependencies, is_pre):
+    pcb_targets = sch_targets = ''
+    pres = BasePreFlight.get_in_use_objs()
+    try:
         for pre in pres:
             tg = pre.get_targets()
             if not tg:
@@ -411,10 +408,17 @@ def generate_makefile(makefile, cfg_file, outputs, kibot_sys=False):
             dependencies[name] = [adapt_file_name(fn) for fn in pre.get_dependencies()]
             is_pre.add(name)
             if pre.is_sch():
-                pre_sch_targets += ' '+name
+                sch_targets += ' '+name
             if pre.is_pcb():
-                pre_pcb_targets += ' '+name
-        # Outputs
+                pcb_targets += ' '+name
+    except KiPlotConfigurationError as e:
+        config_error("In preflight '"+name+"': "+str(e))
+    return pcb_targets, sch_targets
+
+
+def get_out_targets(outputs, ori_names, targets, dependencies, comments):
+    pcb_targets = sch_targets = ''
+    try:
         for out in outputs:
             name = name2make(out.name)
             ori_names[name] = out.name
@@ -426,9 +430,49 @@ def generate_makefile(makefile, cfg_file, outputs, kibot_sys=False):
             if out.comment:
                 comments[name] = out.comment
             if out.is_sch():
-                out_sch_targets += ' '+name
+                sch_targets += ' '+name
             if out.is_pcb():
-                out_pcb_targets += ' '+name
+                pcb_targets += ' '+name
+    except KiPlotConfigurationError as e:
+        config_error("In output '"+name+"': "+str(e))
+    return pcb_targets, sch_targets
+
+
+def generate_makefile(makefile, cfg_file, outputs, kibot_sys=False):
+    cfg_file = os.path.relpath(cfg_file)
+    logger.info('- Creating makefile `{}` from `{}`'.format(makefile, cfg_file))
+    with open(makefile, 'wt') as f:
+        f.write('#!/usr/bin/make\n')
+        f.write('# Automatically generated by KiBot from `{}`\n'.format(cfg_file))
+        fname = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'src', 'kibot'))
+        if kibot_sys or not os.path.isfile(fname):
+            fname = 'kibot'
+        f.write('KIBOT?={}\n'.format(fname))
+        dbg = ''
+        if GS.debug_level > 0:
+            dbg = '-'+'v'*GS.debug_level
+        f.write('DEBUG?={}\n'.format(dbg))
+        f.write('CONFIG={}\n'.format(cfg_file))
+        f.write('SCH={}\n'.format(os.path.relpath(GS.sch_file)))
+        f.write('PCB={}\n'.format(os.path.relpath(GS.pcb_file)))
+        f.write('DEST={}\n'.format(os.path.relpath(GS.out_dir)))
+        f.write('KIBOT_CMD=$(KIBOT) $(DEBUG) -c $(CONFIG) -e $(SCH) -b $(PCB) -d $(DEST)\n')
+        f.write('LOGFILE?=kibot_error.log\n')
+        f.write('\n')
+        # Configure all outputs
+        GS.outputs = outputs
+        for out in outputs:
+            config_output(out)
+        # Get all targets and dependencies
+        targets = OrderedDict()
+        dependencies = OrderedDict()
+        comments = {}
+        ori_names = {}
+        is_pre = set()
+        # Preflights
+        pre_pcb_targets, pre_sch_targets = get_pre_targets(targets, dependencies, is_pre)
+        # Outputs
+        out_pcb_targets, out_sch_targets = get_out_targets(outputs, ori_names, targets, dependencies, comments)
         # all target
         f.write('#\n# Default target\n#\n')
         f.write('all: '+' '.join(targets.keys())+'\n\n')
