@@ -9,6 +9,7 @@ This is somehow compatible with KiBoM.
 """
 import os
 from .gs import GS
+from .misc import W_BADFIELD
 from .optionable import Optionable, BaseOptions
 from .registrable import RegOutput
 from .error import KiPlotConfigurationError
@@ -47,11 +48,15 @@ class BoMColumns(Optionable):
             """ Name to display in the header. The field is used when empty """
             self.join = Optionable
             """ [list(string)|string=''] List of fields to join to this column """
+            self.level = 0
+            """ Used to group columns. The XLSX output uses it to collapse columns """
+            self.comment = ''
+            """ Used as explanation for this column. The XLSX output uses it """
         self._field_example = 'Row'
         self._name_example = 'Line'
 
-    def config(self):
-        super().config()
+    def config(self, parent):
+        super().config(parent)
         if not self.field:
             raise KiPlotConfigurationError("Missing or empty `field` in columns list ({})".format(str(self._tree)))
         # Ensure this is None or a list
@@ -92,8 +97,8 @@ class BoMLinkable(Optionable):
             self.title = 'KiBot Bill of Materials'
             """ BoM title """
 
-    def config(self):
-        super().config()
+    def config(self, parent):
+        super().config(parent)
         # digikey_link
         if isinstance(self.digikey_link, type):
             self.digikey_link = []
@@ -121,8 +126,8 @@ class BoMHTML(BoMLinkable):
             """ Page style. Internal styles: modern-blue, modern-green, modern-red and classic.
                 Or you can provide a CSS file name. Please use .css as file extension. """
 
-    def config(self):
-        super().config()
+    def config(self, parent):
+        super().config(parent)
         # Style
         if not self.style:
             self.style = 'modern-blue'
@@ -155,10 +160,12 @@ class BoMXLSX(BoMLinkable):
             self.max_col_width = 60
             """ [20,999] Maximum column width (characters) """
             self.style = 'modern-blue'
-            """ Head style: modern-blue, modern-green, modern-red and classic. """
+            """ Head style: modern-blue, modern-green, modern-red and classic """
+            self.kicost = False
+            """ Enable KiCost worksheet creation """
 
-    def config(self):
-        super().config()
+    def config(self, parent):
+        super().config(parent)
         # Style
         if not self.style:
             self.style = 'modern-blue'
@@ -200,8 +207,8 @@ class Aggregate(Optionable):
             self.number = 1
             """ Number of boards to build (components multiplier). Use negative to substract """
 
-    def config(self):
-        super().config()
+    def config(self, parent):
+        super().config(parent)
         if not self.file:
             raise KiPlotConfigurationError("Missing or empty `file` in aggregate list ({})".format(str(self._tree)))
         if not self.name:
@@ -231,6 +238,9 @@ class BoMOptions(BaseOptions):
             self.columns = BoMColumns
             """ [list(dict)|list(string)] List of columns to display.
                 Can be just the name of the field """
+            self.cost_extra_columns = BoMColumns
+            """ [list(dict)|list(string)] List of columns to add to the global section of the cost.
+                Can be just the name of the field """
             self.normalize_values = False
             """ Try to normalize the R, L and C values, producing uniform units and prefixes """
             self.normalize_locale = False
@@ -263,6 +273,9 @@ class BoMOptions(BaseOptions):
                 Components which match (comparing *all* fields) will be grouped together.
                 Field names are case-insensitive.
                 If empty: ['Part', 'Part Lib', 'Value', 'Footprint', 'Footprint Lib'] is used """
+            self.group_fields_fallbacks = Optionable
+            """ [list(string)] List of fields to be used when the fields in `group_fields` are empty.
+                The first field in this list is the fallback for the first in `group_fields`, and so on """
             self.component_aliases = ComponentAliases
             """ [list(list(string))] A series of values which are considered to be equivalent for the part name.
                 Each entry is a list of equivalen names. Example: ['c', 'c_small', 'cap' ]
@@ -284,6 +297,8 @@ class BoMOptions(BaseOptions):
             """ A prefix to add to all the references from this project. Used for multiple projects """
             self.source_by_id = False
             """ Generate the `Source BoM` column using the reference ID instead of the project name """
+            self.int_qtys = True
+            """ Component quantities are always expressed as integers. Using the ceil() function """
         self._format_example = 'CSV'
         super().__init__()
 
@@ -323,32 +338,100 @@ class BoMOptions(BaseOptions):
             # Delegate any filter to the variant
             self.variant.set_def_filters(self.exclude_filter, self.dnf_filter, self.dnc_filter)
             self.exclude_filter = self.dnf_filter = self.dnc_filter = None
-            self.variant.config()  # Fill or adjust any detail
+            self.variant.config(self)  # Fill or adjust any detail
 
-    def config(self):
-        super().config()
+    def process_columns_config(self, cols, valid_columns):
+        column_rename = {}
+        join = []
+        if isinstance(cols, type):
+            # If none specified make a list with all the possible columns.
+            # Here are some exceptions:
+            # Ignore the part and footprint library, also sheetpath and the Reference in singular
+            ignore = [ColumnList.COL_PART_LIB_L, ColumnList.COL_FP_LIB_L, ColumnList.COL_SHEETPATH_L,
+                      ColumnList.COL_REFERENCE_L[:-1]]
+            if len(self.aggregate) == 0:
+                ignore.append(ColumnList.COL_SOURCE_BOM_L)
+                if self.number == 1:
+                    # For one board avoid COL_GRP_BUILD_QUANTITY
+                    ignore.append(ColumnList.COL_GRP_BUILD_QUANTITY_L)
+            # Exclude the particular columns
+            columns = [h for h in valid_columns if not h.lower() in ignore]
+            column_levels = [0]*len(columns)
+            column_comments = ['']*len(columns)
+        else:
+            columns = []
+            column_levels = []
+            column_comments = []
+            # Ensure the column names are valid.
+            # Also create the rename and join lists.
+            # Lower case available columns (to check if valid)
+            valid_columns_l = {c.lower(): c for c in valid_columns}
+            logger.debug("Valid columns: {} ({})".format(valid_columns, len(valid_columns)))
+            # Create the different lists
+            for col in cols:
+                if isinstance(col, str):
+                    # Just a string, add to the list of used
+                    new_col = col
+                    new_col_l = new_col.lower()
+                    level = 0
+                    comment = ''
+                else:
+                    # A complete entry
+                    new_col = col.field
+                    new_col_l = new_col.lower()
+                    # A column rename
+                    if col.name:
+                        column_rename[new_col_l] = col.name
+                    # Attach other columns
+                    if col.join:
+                        join.append(col.join)
+                    level = col.level
+                    comment = col.comment
+                # Check this is a valid column
+                if new_col_l not in valid_columns_l:
+                    # The Field_Rename filter can change this situation:
+                    # raise KiPlotConfigurationError('Invalid column name `{}`'.format(new_col))
+                    logger.warning(W_BADFIELD+'Invalid column name `{}`'.format(new_col))
+                columns.append(new_col)
+                column_levels.append(level)
+                column_comments.append(comment)
+        return (columns, column_levels, column_comments, column_rename, join)
+
+    def config(self, parent):
+        super().config(parent)
         self.format = self._guess_format()
+        self._expand_id = 'bom'
+        self._expand_ext = self.format.lower()
         # HTML options
         if self.format == 'html' and isinstance(self.html, type):
             # If no options get the defaults
             self.html = BoMHTML()
-            self.html.config()
+            self.html.config(self)
         # CSV options
         if self.format in ['csv', 'tsv', 'txt'] and isinstance(self.csv, type):
             # If no options get the defaults
             self.csv = BoMCSV()
-            self.csv.config()
+            self.csv.config(self)
         # XLSX options
         if self.format == 'xlsx' and isinstance(self.xlsx, type):
             # If no options get the defaults
             self.xlsx = BoMXLSX()
-            self.xlsx.config()
+            self.xlsx.config(self)
         # group_fields
         if isinstance(self.group_fields, type):
             self.group_fields = ColumnList.DEFAULT_GROUPING
         else:
             # Make the grouping fields lowercase
             self.group_fields = [f.lower() for f in self.group_fields]
+        # group_fields_fallbacks
+        if isinstance(self.group_fields_fallbacks, type):
+            self.group_fields_fallbacks = []
+        else:
+            # Make the grouping fields lowercase
+            self.group_fields_fallbacks = [f.lower() for f in self.group_fields_fallbacks]
+        # Fill with None if needed
+        if len(self.group_fields_fallbacks) < len(self.group_fields):
+            self.group_fields_fallbacks.extend([None]*(len(self.group_fields)-len(self.group_fields_fallbacks)))
         # component_aliases
         if isinstance(self.component_aliases, type):
             self.component_aliases = DEFAULT_ALIASES
@@ -373,49 +456,11 @@ class BoMOptions(BaseOptions):
         if isinstance(self.aggregate, type):
             self.aggregate = []
         # Columns
-        self.column_rename = {}
-        self.join = []
         valid_columns = self._get_columns()
-        if isinstance(self.columns, type):
-            # If none specified make a list with all the possible columns.
-            # Here are some exceptions:
-            # Ignore the part and footprint library, also sheetpath and the Reference in singular
-            ignore = [ColumnList.COL_PART_LIB_L, ColumnList.COL_FP_LIB_L, ColumnList.COL_SHEETPATH_L,
-                      ColumnList.COL_REFERENCE_L[:-1]]
-            if len(self.aggregate) == 0:
-                ignore.append(ColumnList.COL_SOURCE_BOM_L)
-                if self.number == 1:
-                    # For one board avoid COL_GRP_BUILD_QUANTITY
-                    ignore.append(ColumnList.COL_GRP_BUILD_QUANTITY_L)
-            # Exclude the particular columns
-            self.columns = [h for h in valid_columns if not h.lower() in ignore]
-        else:
-            # Ensure the column names are valid.
-            # Also create the rename and join lists.
-            # Lower case available columns (to check if valid)
-            valid_columns_l = {c.lower(): c for c in valid_columns}
-            logger.debug("Valid columns: {} ({})".format(valid_columns, len(valid_columns)))
-            # Create the different lists
-            columns = []
-            for col in self.columns:
-                if isinstance(col, str):
-                    # Just a string, add to the list of used
-                    new_col = col
-                else:
-                    # A complete entry
-                    new_col = col.field
-                    # A column rename
-                    if col.name:
-                        self.column_rename[col.field.lower()] = col.name
-                    # Attach other columns
-                    if col.join:
-                        self.join.append(col.join)
-                # Check this is a valid column
-                if new_col.lower() not in valid_columns_l:
-                    raise KiPlotConfigurationError('Invalid column name `{}`'.format(new_col))
-                columns.append(new_col)
-            # This is the ordered list with the case style defined by the user
-            self.columns = columns
+        (self.columns, self.column_levels, self.column_comments, self.column_rename,
+         self.join) = self.process_columns_config(self.columns, valid_columns)
+        (self.columns_ce, self.column_levels_ce, self.column_comments_ce, self.column_rename_ce,
+         self.join_ce) = self.process_columns_config(self.cost_extra_columns, valid_columns)
 
     def aggregate_comps(self, comps):
         self.qtys = {GS.sch_basename: self.number}
@@ -434,9 +479,8 @@ class BoMOptions(BaseOptions):
             comps.extend(new_comps)
             prj.source = os.path.basename(prj.file)
 
-    def run(self, output_dir):
+    def run(self, output):
         format = self.format.lower()
-        output = self.expand_filename_sch(output_dir, self.output, 'bom', format)
         # Add some info needed for the output to the config object.
         # So all the configuration is contained in one object.
         self.source = GS.sch_basename
@@ -459,7 +503,7 @@ class BoMOptions(BaseOptions):
         apply_fitted_filter(comps, self.dnf_filter)
         apply_fixed_filter(comps, self.dnc_filter)
         # Apply the variant
-        self.variant.filter(comps)
+        comps = self.variant.filter(comps)
         # We add the main project to the aggregate list so do_bom sees a complete list
         base_sch = Aggregate()
         base_sch.file = GS.sch_file
@@ -482,8 +526,8 @@ class BoMOptions(BaseOptions):
                 c.ref = c.ref[l_id:]
                 c.ref_id = ''
 
-    def get_targets(self, parent, out_dir):
-        return [self.expand_filename_sch(out_dir, self.output, 'bom', self.format.lower())]
+    def get_targets(self, out_dir):
+        return [self._parent.expand_filename(out_dir, self.output)]
 
 
 @output_class
