@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
-# Copyright (c) 2020 Salvador E. Tropea
-# Copyright (c) 2020 Instituto Nacional de Tecnología Industrial
+# Copyright (c) 2020-2021 Salvador E. Tropea
+# Copyright (c) 2020-2021 Instituto Nacional de Tecnología Industrial
 # License: GPL-3.0
 # Project: KiBot (formerly KiPlot)
 from pcbnew import GERBER_WRITER
@@ -11,13 +11,14 @@ from .macros import macros, document, output_class  # noqa: F401
 class Gerb_DrillOptions(AnyDrill):
     def __init__(self):
         super().__init__()
+        self._ext = 'gbr'
 
     def _configure_writer(self, board, offset):
         drill_writer = GERBER_WRITER(board)
         # hard coded in UI?
         drill_writer.SetFormat(5)
         drill_writer.SetOptions(offset)
-        return drill_writer, 'gbr'
+        return drill_writer
 
 
 @output_class
