@@ -55,8 +55,8 @@ class KiBoMColumns(Optionable):
         self._field_example = 'Row'
         self._name_example = 'Line'
 
-    def config(self):
-        super().config()
+    def config(self, parent):
+        super().config(parent)
         if not self.field:
             raise KiPlotConfigurationError("Missing or empty `field` in columns list ({})".format(str(self._tree)))
         if isinstance(self.join, type):
@@ -207,8 +207,8 @@ class KiBoMConfig(Optionable):
         logger.debug('Output from command:\n'+cmd_output.decode())
         return columns
 
-    def config(self):
-        super().config()
+    def config(self, parent):
+        super().config(parent)
         # digikey_link
         if isinstance(self.digikey_link, type):
             self.digikey_link = None
@@ -341,9 +341,10 @@ class KiBoMOptions(BaseOptions):
             self.format = 'HTML'
             """ [HTML,CSV,XML,XLSX] format for the BoM """
         super().__init__()
+        self._expand_id = 'bom'
 
-    def config(self):
-        super().config()
+    def config(self, parent):
+        super().config(parent)
         if isinstance(self.conf, type):
             self.conf = 'bom.ini'
         elif isinstance(self.conf, str):
@@ -354,25 +355,28 @@ class KiBoMOptions(BaseOptions):
             conf = os.path.abspath(os.path.join(GS.out_dir, CONFIG_FILENAME))
             self.conf.save(conf)
             self.conf = conf
+        self._expand_ext = self.format.lower()
 
-    def get_targets(self, parent, out_dir):
+    def get_targets(self, out_dir):
         if self.output:
-            return [self.expand_filename_sch(out_dir, self.output, 'bom', self.format.lower())]
-        logger.warning(W_EXTNAME+'{} uses a name generated by the external tool.'.format(parent))
+            return [self.expand_filename(out_dir, self.output, 'bom', self.format.lower())]
+        logger.warning(W_EXTNAME+'{} uses a name generated by the external tool.'.format(self._parent))
         logger.warning(W_EXTNAME+'Please use a name generated by KiBot or specify the name explicitly.')
         return []
 
-    def run(self, output_dir):
+    def run(self, name):
         check_script(CMD_KIBOM, URL_KIBOM, '1.8.0')
         format = self.format.lower()
         prj = GS.sch_no_ext
         config = os.path.join(GS.sch_dir, self.conf)
         if self.output:
             force_output = True
-            output = self.expand_filename_sch(output_dir, self.output, 'bom', format)
+            output = name
+            output_dir = os.path.dirname(name)
         else:
             force_output = False
             output = os.path.basename(prj)+'.'+format
+            output_dir = name
         logger.debug('Doing BoM, format {} prj: {} config: {} output: {}'.format(format, prj, config, output))
         cmd = [CMD_KIBOM,
                '-n', str(self.number),
